@@ -60,7 +60,7 @@ using namespace Kleo;
 inline QIcon loadIcon(const QString &s)
 {
     QString ss = s;
-    const static QRegularExpression reg(QRegularExpression(QLatin1String("[^a-zA-Z0-9_]")));
+    const static QRegularExpression reg(QRegularExpression(QLatin1StringView("[^a-zA-Z0-9_]")));
     return QIcon::fromTheme(ss.replace(reg, QStringLiteral("-")));
 }
 
@@ -280,7 +280,7 @@ bool offerEntryForConfiguration(QGpgME::CryptoConfigEntry *entry)
                                      : QGpgME::CryptoConfigEntry::Level_Expert + 1;
     // we ignore the group when looking up entries to exclude because entries
     // are uniquely identified by their name and their component
-    const auto entryId = entry->path().replace(entryPathGroupSegmentRegexp, QLatin1String{"/"}).toLower();
+    const auto entryId = entry->path().replace(entryPathGroupSegmentRegexp, QLatin1StringView{"/"}).toLower();
     return (entry->level() <= maxEntryLevel) && (entriesToExclude.find(entryId) == entriesToExclude.end());
 }
 
@@ -536,7 +536,7 @@ QString Kleo::CryptoConfigEntryGUI::description() const
               "first character to upper-case. It is this behaviour you can "
               "control for your language with this translation.",
               "yes")
-        == QLatin1String("yes")) {
+        == QLatin1StringView("yes")) {
         descr[0] = descr[0].toUpper();
     }
     return descr;
@@ -628,7 +628,7 @@ void Kleo::CryptoConfigEntryDebugLevel::doSave()
 {
     const unsigned int idx = mComboBox->currentIndex();
     if (idx < numDebugLevels) {
-        mEntry->setStringValue(QLatin1String(debugLevels[idx].name));
+        mEntry->setStringValue(QLatin1StringView(debugLevels[idx].name));
     } else {
         mEntry->setStringValue(QString());
     }
@@ -638,7 +638,7 @@ void Kleo::CryptoConfigEntryDebugLevel::doLoad()
 {
     const QString str = mEntry->stringValue();
     for (unsigned int i = 0; i < numDebugLevels; ++i) {
-        if (str == QLatin1String(debugLevels[i].name)) {
+        if (str == QLatin1StringView(debugLevels[i].name)) {
             mComboBox->setCurrentIndex(i);
             return;
         }
