@@ -37,7 +37,6 @@
 #include <QLocale>
 #include <QRegularExpression>
 #include <QString>
-#include <QTextDocument> // for Qt::escape
 
 #include <gpgme++/importresult.h>
 #include <gpgme++/key.h>
@@ -74,20 +73,24 @@ QIcon iconForValidity(const UserID &userId)
 
 QIcon Formatting::IconProvider::icon(const GpgME::Key &key) const
 {
-    if (usage.canEncrypt() && !Kleo::canBeUsedForEncryption(key)) {
+    return icon(key.userID(0));
+}
+
+QIcon Formatting::IconProvider::icon(const GpgME::UserID &userID) const
+{
+    if (usage.canEncrypt() && !Kleo::canBeUsedForEncryption(userID.parent())) {
         return Formatting::errorIcon();
     }
-    if (usage.canSign() && !Kleo::canBeUsedForSigning(key)) {
+    if (usage.canSign() && !Kleo::canBeUsedForSigning(userID.parent())) {
         return Formatting::errorIcon();
     }
-    if (key.isBad()) {
+    if (userID.parent().isBad() || userID.isBad()) {
         return Formatting::errorIcon();
     }
-    const auto primaryUserId = key.userID(0);
-    if (Kleo::isRevokedOrExpired(primaryUserId)) {
+    if (Kleo::isRevokedOrExpired(userID)) {
         return Formatting::errorIcon();
     }
-    return iconForValidity(primaryUserId);
+    return iconForValidity(userID);
 }
 
 QIcon Formatting::IconProvider::icon(const KeyGroup &group) const
@@ -225,7 +228,7 @@ QString Formatting::prettyKeyID(const char *id)
     if (!id) {
         return QString();
     }
-    return QLatin1String("0x") + QString::fromLatin1(id).toUpper();
+    return QLatin1StringView("0x") + QString::fromLatin1(id).toUpper();
 }
 
 QString Formatting::prettyNameAndEMail(const UserID &uid)
@@ -322,9 +325,9 @@ QString format_keytype(const Key &key)
 {
     const Subkey subkey = key.subkey(0);
     if (key.hasSecret()) {
-        return i18n("%1-bit %2 (secret key available)", subkey.length(), QLatin1String(subkey.publicKeyAlgorithmAsString()));
+        return i18n("%1-bit %2 (secret key available)", subkey.length(), QLatin1StringView(subkey.publicKeyAlgorithmAsString()));
     } else {
-        return i18n("%1-bit %2", subkey.length(), QLatin1String(subkey.publicKeyAlgorithmAsString()));
+        return i18n("%1-bit %2", subkey.length(), QLatin1StringView(subkey.publicKeyAlgorithmAsString()));
     }
 }
 
@@ -335,7 +338,7 @@ QString format_subkeytype(const Subkey &subkey)
     if (algo == Subkey::AlgoECC || algo == Subkey::AlgoECDSA || algo == Subkey::AlgoECDH || algo == Subkey::AlgoEDDSA) {
         return QString::fromStdString(subkey.algoName());
     }
-    return i18n("%1-bit %2", subkey.length(), QLatin1String(subkey.publicKeyAlgorithmAsString()));
+    return i18n("%1-bit %2", subkey.length(), QLatin1StringView(subkey.publicKeyAlgorithmAsString()));
 }
 
 QString format_keyusage(const Key &key)
@@ -357,7 +360,7 @@ QString format_keyusage(const Key &key)
     if (Kleo::keyHasAuthenticate(key)) {
         capabilities.push_back(i18n("SSH Authentication"));
     }
-    return capabilities.join(QLatin1String(", "));
+    return capabilities.join(QLatin1StringView(", "));
 }
 
 QString format_subkeyusage(const Subkey &subkey)
@@ -379,7 +382,7 @@ QString format_subkeyusage(const Subkey &subkey)
     if (subkey.canAuthenticate()) {
         capabilities.push_back(i18n("SSH Authentication"));
     }
-    return capabilities.join(QLatin1String(", "));
+    return capabilities.join(QLatin1StringView(", "));
 }
 
 static QString time_t2string(time_t t)
@@ -390,12 +393,12 @@ static QString time_t2string(time_t t)
 
 static QString make_red(const QString &txt)
 {
-    return QLatin1String("<font color=\"red\">") + txt.toHtmlEscaped() + QLatin1String("</font>");
+    return QLatin1StringView("<font color=\"red\">") + txt.toHtmlEscaped() + QLatin1String("</font>");
 }
 
 }
 
-QString Formatting::toolTip(const Key &key, int flags)
+static QString toolTipInternal(const GpgME::Key &key, const GpgME::UserID &userID, int flags)
 {
     if (flags == 0 || (key.protocol() != GpgME::CMS && key.protocol() != GpgME::OpenPGP)) {
         return QString();
@@ -404,29 +407,41 @@ QString Formatting::toolTip(const Key &key, int flags)
     const Subkey subkey = key.subkey(0);
 
     QString result;
-    if (flags & Validity) {
+    if (flags & Formatting::Validity) {
         if (key.protocol() == GpgME::OpenPGP || (key.keyListMode() & Validate)) {
-            if (key.isRevoked()) {
+            if (userID.isRevoked() || key.isRevoked()) {
                 result = make_red(i18n("Revoked"));
             } else if (key.isExpired()) {
                 result = make_red(i18n("Expired"));
             } else if (key.isDisabled()) {
                 result = i18n("Disabled");
             } else if (key.keyListMode() & GpgME::Validate) {
-                unsigned int fullyTrusted = 0;
-                for (const auto &uid : key.userIDs()) {
-                    if (uid.validity() >= UserID::Validity::Full) {
-                        fullyTrusted++;
-                    }
-                }
-                if (fullyTrusted == key.numUserIDs()) {
-                    result = i18n("All User-IDs are certified.");
-                    const auto compliance = complianceStringForKey(key);
-                    if (!compliance.isEmpty()) {
-                        result += QStringLiteral("<br>") + compliance;
+                if (!userID.isNull()) {
+                    if (userID.validity() >= UserID::Validity::Full) {
+                        result = i18n("User-ID is certified.");
+                        const auto compliance = Formatting::complianceStringForUserID(userID);
+                        if (!compliance.isEmpty()) {
+                            result += QStringLiteral("<br>") + compliance;
+                        }
+                    } else {
+                        result = i18n("User-ID is not certified.");
                     }
                 } else {
-                    result = i18np("One User-ID is not certified.", "%1 User-IDs are not certified.", key.numUserIDs() - fullyTrusted);
+                    unsigned int fullyTrusted = 0;
+                    for (const auto &uid : key.userIDs()) {
+                        if (uid.validity() >= UserID::Validity::Full) {
+                            fullyTrusted++;
+                        }
+                    }
+                    if (fullyTrusted == key.numUserIDs()) {
+                        result = i18n("All User-IDs are certified.");
+                        const auto compliance = Formatting::complianceStringForKey(key);
+                        if (!compliance.isEmpty()) {
+                            result += QStringLiteral("<br>") + compliance;
+                        }
+                    } else {
+                        result = i18np("One User-ID is not certified.", "%1 User-IDs are not certified.", key.numUserIDs() - fullyTrusted);
+                    }
                 }
             } else {
                 result = i18n("The validity cannot be checked at the moment.");
@@ -435,76 +450,80 @@ QString Formatting::toolTip(const Key &key, int flags)
             result = i18n("The validity cannot be checked at the moment.");
         }
     }
-    if (flags == Validity) {
+    if (flags == Formatting::Validity) {
         return result;
     }
 
-    result += QLatin1String("<table border=\"0\">");
+    result += QLatin1StringView("<table border=\"0\">");
     if (key.protocol() == GpgME::CMS) {
-        if (flags & SerialNumber) {
+        if (flags & Formatting::SerialNumber) {
             result += format_row(i18n("Serial number"), key.issuerSerial());
         }
-        if (flags & Issuer) {
+        if (flags & Formatting::Issuer) {
             result += format_row(i18n("Issuer"), key.issuerName());
         }
     }
-    if (flags & UserIDs) {
-        const std::vector<UserID> uids = key.userIDs();
-        if (!uids.empty()) {
-            result += format_row(key.protocol() == GpgME::CMS ? i18n("Subject") : i18n("User-ID"), prettyUserID(uids.front()));
-        }
-        if (uids.size() > 1) {
-            for (auto it = uids.begin() + 1, end = uids.end(); it != end; ++it) {
-                if (!it->isRevoked() && !it->isInvalid()) {
-                    result += format_row(i18n("a.k.a."), prettyUserID(*it));
+    if (flags & Formatting::UserIDs) {
+        if (userID.isNull()) {
+            const std::vector<UserID> uids = key.userIDs();
+            if (!uids.empty()) {
+                result += format_row(key.protocol() == GpgME::CMS ? i18n("Subject") : i18n("User-ID"), Formatting::prettyUserID(uids.front()));
+            }
+            if (uids.size() > 1) {
+                for (auto it = uids.begin() + 1, end = uids.end(); it != end; ++it) {
+                    if (!it->isRevoked() && !it->isInvalid()) {
+                        result += format_row(i18n("a.k.a."), Formatting::prettyUserID(*it));
+                    }
                 }
             }
+        } else {
+            result += format_row(key.protocol() == GpgME::CMS ? i18n("Subject") : i18n("User-ID"), Formatting::prettyUserID(userID));
         }
     }
-    if (flags & ExpiryDates) {
+    if (flags & Formatting::ExpiryDates) {
         result += format_row(i18n("Valid from"), time_t2string(subkey.creationTime()));
 
         if (!subkey.neverExpires()) {
             result += format_row(i18n("Valid until"), time_t2string(subkey.expirationTime()));
         }
     }
-    if (flags & CertificateType) {
+
+    if (flags & Formatting::CertificateType) {
         result += format_row(i18n("Type"), format_keytype(key));
     }
-    if (flags & CertificateUsage) {
+    if (flags & Formatting::CertificateUsage) {
         result += format_row(i18n("Usage"), format_keyusage(key));
     }
-    if (flags & KeyID) {
+    if (flags & Formatting::KeyID) {
         result += format_row(i18n("Key-ID"), QString::fromLatin1(key.shortKeyID()));
     }
-    if (flags & Fingerprint) {
+    if (flags & Formatting::Fingerprint) {
         result += format_row(i18n("Fingerprint"), key.primaryFingerprint());
     }
-    if (flags & OwnerTrust) {
+    if (flags & Formatting::OwnerTrust) {
         if (key.protocol() == GpgME::OpenPGP) {
-            result += format_row(i18n("Certification trust"), ownerTrustShort(key));
+            result += format_row(i18n("Certification trust"), Formatting::ownerTrustShort(key));
         } else if (key.isRoot()) {
-            result += format_row(i18n("Trusted issuer?"), key.userID(0).validity() == UserID::Ultimate ? i18n("Yes") : i18n("No"));
+            result += format_row(i18n("Trusted issuer?"), (userID.isNull() ? key.userID(0) : userID).validity() == UserID::Ultimate ? i18n("Yes") : i18n("No"));
         }
     }
-
-    if (flags & StorageLocation) {
+    if (flags & Formatting::StorageLocation) {
         if (const char *card = subkey.cardSerialNumber()) {
             result += format_row(i18n("Stored"), i18nc("stored...", "on SmartCard with serial no. %1", QString::fromUtf8(card)));
         } else {
             result += format_row(i18n("Stored"), i18nc("stored...", "on this computer"));
         }
     }
-    if (flags & Subkeys) {
+    if (flags & Formatting::Subkeys) {
         for (const auto &sub : key.subkeys()) {
-            result += QLatin1String("<hr/>");
+            result += QLatin1StringView("<hr/>");
             result += format_row(i18n("Subkey"), sub.fingerprint());
             if (sub.isRevoked()) {
                 result += format_row(i18n("Status"), i18n("Revoked"));
             } else if (sub.isExpired()) {
                 result += format_row(i18n("Status"), i18n("Expired"));
             }
-            if (flags & ExpiryDates) {
+            if (flags & Formatting::ExpiryDates) {
                 result += format_row(i18n("Valid from"), time_t2string(sub.creationTime()));
 
                 if (!sub.neverExpires()) {
@@ -512,13 +531,13 @@ QString Formatting::toolTip(const Key &key, int flags)
                 }
             }
 
-            if (flags & CertificateType) {
+            if (flags & Formatting::CertificateType) {
                 result += format_row(i18n("Type"), format_subkeytype(sub));
             }
-            if (flags & CertificateUsage) {
+            if (flags & Formatting::CertificateUsage) {
                 result += format_row(i18n("Usage"), format_subkeyusage(sub));
             }
-            if (flags & StorageLocation) {
+            if (flags & Formatting::StorageLocation) {
                 if (const char *card = sub.cardSerialNumber()) {
                     result += format_row(i18n("Stored"), i18nc("stored...", "on SmartCard with serial no. %1", QString::fromUtf8(card)));
                 } else {
@@ -527,11 +546,16 @@ QString Formatting::toolTip(const Key &key, int flags)
             }
         }
     }
-    result += QLatin1String("</table>");
+    result += QLatin1StringView("</table>");
 
     return result;
 }
 
+QString Formatting::toolTip(const Key &key, int flags)
+{
+    return toolTipInternal(key, UserID(), flags);
+}
+
 namespace
 {
 template<typename Container>
@@ -573,6 +597,12 @@ QString Formatting::toolTip(const KeyGroup &group, int flags)
         return i18nc("@info:tooltip", "This group does not contain any keys.");
     }
 
+    if (Kleo::any_of(keys, [](const auto &key) {
+            return !key.hasEncrypt();
+        })) {
+        return i18nc("@info:tooltip", "Some of the certificates in this group cannot be used for encryption. Using this group can lead to unexpected results.");
+    }
+
     const QString validity = (flags & Validity) ? getValidityStatement(keys) : QString();
     if (flags == Validity) {
         return validity;
@@ -594,17 +624,23 @@ QString Formatting::toolTip(const KeyGroup &group, int flags)
     {
         auto it = keys.cbegin();
         for (unsigned int i = 0; i < numKeysForTooltip; ++i, ++it) {
-            result.push_back(QLatin1String("<br>") + Formatting::summaryLine(*it).toHtmlEscaped());
+            result.push_back(QLatin1StringView("<br>") + Formatting::summaryLine(*it).toHtmlEscaped());
         }
     }
     if (keys.size() > numKeysForTooltip) {
-        result.push_back(QLatin1String("<br>") + i18ncp("this follows a list of keys", "and 1 more key", "and %1 more keys", keys.size() - numKeysForTooltip));
+        result.push_back(QLatin1StringView("<br>")
+                         + i18ncp("this follows a list of keys", "and 1 more key", "and %1 more keys", keys.size() - numKeysForTooltip));
     }
     result.push_back(QStringLiteral("</p>"));
 
     return result.join(QLatin1Char('\n'));
 }
 
+QString Formatting::toolTip(const UserID &userID, int flags)
+{
+    return toolTipInternal(userID.parent(), userID, flags);
+}
+
 //
 // Creation and Expiration
 //
@@ -958,7 +994,7 @@ QString Formatting::formatKeyLink(const Key &key)
     if (key.isNull()) {
         return QString();
     }
-    return QStringLiteral("<a href=\"key:%1\">%2</a>").arg(QLatin1String(key.primaryFingerprint()), Formatting::prettyName(key));
+    return QStringLiteral("<a href=\"key:%1\">%2</a>").arg(QLatin1StringView(key.primaryFingerprint()), Formatting::prettyName(key));
 }
 
 QString Formatting::formatForComboBox(const GpgME::Key &key)
@@ -968,7 +1004,23 @@ QString Formatting::formatForComboBox(const GpgME::Key &key)
     if (!mail.isEmpty()) {
         mail = QLatin1Char('<') + mail + QLatin1Char('>');
     }
-    return i18nc("name, email, key id", "%1 %2 (%3)", name, mail, QLatin1String(key.shortKeyID())).simplified();
+    return i18nc("name, email, key id", "%1 %2 (%3)", name, mail, QLatin1StringView(key.shortKeyID())).simplified();
+}
+
+QString Formatting::nameAndEmailForSummaryLine(const UserID &id)
+{
+    Q_ASSERT(!id.isNull());
+
+    const QString email = Formatting::prettyEMail(id);
+    const QString name = Formatting::prettyName(id);
+
+    if (name.isEmpty()) {
+        return email;
+    } else if (email.isEmpty()) {
+        return name;
+    } else {
+        return QStringLiteral("%1 <%2>").arg(name, email);
+    }
 }
 
 QString Formatting::nameAndEmailForSummaryLine(const Key &key)
@@ -1111,7 +1163,20 @@ QString Formatting::usageString(const Subkey &sub)
     if (sub.canAuthenticate()) {
         usageStrings << i18n("Authenticate");
     }
-    return usageStrings.join(QLatin1String(", "));
+    if (sub.canRenc()) {
+        usageStrings << i18nc("Means 'Additional Decryption Subkey'; Don't try translating that, though.", "ADSK");
+    }
+    return usageStrings.join(QLatin1StringView(", "));
+}
+
+QString Formatting::summaryLine(const UserID &id)
+{
+    return i18nc("name <email> (validity, protocol, creation date)",
+                 "%1 (%2, %3, created: %4)",
+                 nameAndEmailForSummaryLine(id),
+                 Formatting::complianceStringShort(id),
+                 displayName(id.parent().protocol()),
+                 Formatting::creationDateString(id.parent()));
 }
 
 QString Formatting::summaryLine(const Key &key)
@@ -1233,7 +1298,7 @@ bool Formatting::uidsHaveFullValidity(const Key &key)
 QString Formatting::complianceMode()
 {
     const auto complianceValue = getCryptoConfigStringValue("gpg", "compliance");
-    return complianceValue == QLatin1String("gnupg") ? QString() : complianceValue;
+    return complianceValue == QLatin1StringView("gnupg") ? QString() : complianceValue;
 }
 
 bool Formatting::isKeyDeVs(const GpgME::Key &key)
@@ -1253,6 +1318,46 @@ QString Formatting::complianceStringForKey(const GpgME::Key &key)
     return QString();
 }
 
+QString Formatting::complianceStringForUserID(const GpgME::UserID &userID)
+{
+    // There will likely be more in the future for other institutions
+    // for now we only have DE-VS
+    if (DeVSCompliance::isCompliant()) {
+        return isRemoteKey(userID.parent()) //
+            ? i18nc("@info the compliance of the key with certain requirements is unknown", "unknown")
+            : DeVSCompliance::name(DeVSCompliance::userIDIsCompliant(userID));
+    }
+    return QString();
+}
+
+QString Formatting::complianceStringShort(const GpgME::UserID &id)
+{
+    if (DeVSCompliance::isCompliant() && DeVSCompliance::userIDIsCompliant(id)) {
+        return QStringLiteral("★ ") + DeVSCompliance::name(true);
+    }
+    const bool keyValidityChecked = (id.parent().keyListMode() & GpgME::Validate);
+    if (keyValidityChecked && id.validity() >= UserID::Full) {
+        return i18nc("As in 'this user ID is valid.'", "certified");
+    }
+    if (id.parent().isExpired() || isExpired(id)) {
+        return i18n("expired");
+    }
+    if (id.parent().isRevoked() || id.isRevoked()) {
+        return i18n("revoked");
+    }
+    if (id.parent().isDisabled()) {
+        return i18n("disabled");
+    }
+    if (id.parent().isInvalid() || id.isInvalid()) {
+        return i18n("invalid");
+    }
+    if (keyValidityChecked) {
+        return i18nc("As in 'this user ID is not certified'", "not certified");
+    }
+
+    return i18nc("The validity of this user ID has not been/could not be checked", "not checked");
+}
+
 QString Formatting::complianceStringShort(const GpgME::Key &key)
 {
     if (DeVSCompliance::isCompliant() && DeVSCompliance::keyIsCompliant(key)) {
@@ -1397,3 +1502,32 @@ QString Formatting::errorAsString(const GpgME::Error &error)
     return QString::fromLocal8Bit(error.asString());
 #endif
 }
+
+QString Formatting::prettyAlgorithmName(const std::string &algorithm)
+{
+    static const std::map<std::string, QString> displayNames = {
+        {"brainpoolP256r1", i18nc("@info", "ECC (Brainpool P-256)")},
+        {"brainpoolP384r1", i18nc("@info", "ECC (Brainpool P-384)")},
+        {"brainpoolP512r1", i18nc("@info", "ECC (Brainpool P-512)")},
+        {"curve25519", i18nc("@info", "ECC (Curve25519)")},
+        {"curve448", i18nc("@info", "ECC (Curve448)")},
+        {"ed25519", i18nc("@info", "ECC (Ed25519)")},
+        {"ed448", i18nc("@info", "ECC (Ed448)")},
+        {"cv25519", i18nc("@info", "ECC (Cv25519)")},
+        {"cv448", i18nc("@info", "ECC (Cv448)")},
+        {"nistp256", i18nc("@info", "ECC (NIST P-256)")},
+        {"nistp384", i18nc("@info", "ECC (NIST P-384)")},
+        {"nistp521", i18nc("@info", "ECC (NIST P-521)")},
+        {"rsa2048", i18nc("@info", "RSA 2048")},
+        {"rsa3072", i18nc("@info", "RSA 3072")},
+        {"rsa4096", i18nc("@info", "RSA 4096")},
+        {"dsa1024", i18nc("@info", "DSA 1024")},
+        {"dsa2048", i18nc("@info", "DSA 2048")},
+        {"elg1024", i18nc("@info", "Elgamal 1024")},
+        {"elg2048", i18nc("@info", "Elgamal 2048")},
+        {"elg3072", i18nc("@info", "Elgamal 3072")},
+        {"elg4096", i18nc("@info", "Elgamal 4096")},
+    };
+    const auto it = displayNames.find(algorithm);
+    return (it != displayNames.end()) ? it->second : i18nc("@info", "Unknown algorithm");
+}
